/*
Copyright (c) 2010, Paul Furgale and Chi Hay Tong
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are 
met:

* Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.
* The names of its contributors may not be used to endorse or promote 
  products derived from this software without specific prior written 
  permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef ASRL_GPU_AREA_TESTS_HPP
#define ASRL_GPU_AREA_TESTS_HPP

#include "GpuIntegralImageProcessor.hpp"
#include "gpu_area.h"
#include <opencv/highgui.h>
#include <opencv/highgui.hpp>

/// A test suite for the timer class
BOOST_AUTO_TEST_SUITE(asrl_area)

BOOST_AUTO_TEST_CASE(test_area_lookup) 
{
  // create an image and fill it with zeros.
  cv::Mat image = cv::Mat::zeros(64,64,CV_8UC1);

  // Now fill in a 3x5 block.
  int cx = 30;
  int cy = 30;
  
  for(int r = -1; r < 2; r++)
    {
      for(int c = -2; c < 3; c++)
	{
	  // When 255 is converted to a float, it will be 1.f.
	  image.ptr(cy + r)[cx + c] = 255;
	}
    }

  // Create an integral image.
  asrl::GpuIntegralImageProcessor iiProc(image.cols, image.rows);
  asrl::GpuIntegralImage iimage(image.cols, image.rows);

  iiProc.process(image, iimage);

  float area = asrl::iiAreaLookupC(iimage, (float)cx, (float)cy, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 15.f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx-0.5f, (float)cy, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 13.5f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx-1, (float)cy, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 12.f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx-0.5f, (float)cy, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 13.5f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx+1, (float)cy, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 12.f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx, (float)cy-1, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 10.f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx, (float)cy-0.5f, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 12.5f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx, (float)cy+1, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 10.f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx, (float)cy+0.5f, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 12.5f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx-1, (float)cy-1, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 8.f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx-0.5f, (float)cy-0.5f, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 11.25f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx+1, (float)cy+1, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 8.f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx+0.5f, (float)cy+0.5f, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 11.25f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx-1, (float)cy+1, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 8.f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx+1, (float)cy-1, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 8.f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx-0.5f, (float)cy+0.5f, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 11.25f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx+0.5f, (float)cy-0.5f, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 11.25f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx-1, (float)cy+1, 5.f, 3.f);
  BOOST_CHECK_CLOSE(area, 8.f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx, (float)cy, 3.f, 3.f);
  BOOST_CHECK_CLOSE(area, 9.f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx, (float)cy, 3.f, 4.5f);
  BOOST_CHECK_CLOSE(area, 9.f,0.001f);

  area = asrl::iiAreaLookupC(iimage, (float)cx, (float)cy, 4.5f, 3.f);
  BOOST_CHECK_CLOSE(area, 13.5f,0.001f);
}


BOOST_AUTO_TEST_SUITE_END()

#endif // ASRL_GPU_AREA_TESTS_HPP
