/*
Copyright (c) 2010, Paul Furgale and Chi Hay Tong
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are 
met:

* Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.
* The names of its contributors may not be used to endorse or promote 
  products derived from this software without specific prior written 
  permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "gpu_globals.h"
#include <iostream>
#include "assert_macros.hpp"
#include "GpuSurfOctave.hpp"

namespace asrl {

  bool s_initialized = false;
  int s_initWidth = -1;
  int s_initHeight = -1;	// texturing the integral image
  __constant__ SurfOctaveParameters d_octave_params[ASRL_SURF_MAX_OCTAVES];
  __constant__ float d_hessian_scale[ASRL_SURF_MAX_INTERVALS*ASRL_SURF_MAX_OCTAVES];

  void init_globals(int imWidth, int imHeight, std::vector<asrl::GpuSurfOctave> & octaves)
  {
    if(imWidth == s_initWidth && imHeight == s_initHeight)
      return;

    float hessian_scale[ASRL_SURF_MAX_OCTAVES*ASRL_SURF_MAX_INTERVALS];
    SurfOctaveParameters params[ASRL_SURF_MAX_OCTAVES];
    for(unsigned r = 0; r < octaves.size(); r++)
      {
	params[r] = (SurfOctaveParameters)octaves[r];
	for(int c = 0; c < octaves[r].intervals(); c++)
	  {
	    hessian_scale[r*ASRL_SURF_MAX_INTERVALS + c] = octaves[r].scales()[c];
	  }
      }

    //std::cout << "Initializing FH globals: " << s_initWidth << " != " << imWidth << ", " << s_initHeight << " != " << imHeight << "\n";
    cudaError_t err = cudaMemcpyToSymbol( d_hessian_scale, hessian_scale, ASRL_SURF_MAX_OCTAVES*ASRL_SURF_MAX_INTERVALS*sizeof(float));
    ASRL_ASSERT_EQ(err,cudaSuccess, "Unable to copy cuda hessian scale parameter array: " << cudaGetErrorString(err));  

    err = cudaMemcpyToSymbol(d_octave_params, params, ASRL_SURF_MAX_OCTAVES*sizeof(SurfOctaveParameters));
    ASRL_ASSERT_EQ(err,cudaSuccess, "Unable to copy cuda hessian parameter array: " << cudaGetErrorString(err));  

    s_initWidth = imWidth;
    s_initHeight = imHeight;
    s_initialized = true;

  }

} // namespace asrl

